/* naytto.cpp */
/****************************************************************************/
/*
**        N A Y T T O . C P P
**
** Nytt-luokan toteutus
**
**  Tekijt:          Vesa Lappalainen
**                    Ohjelmointikurssi 1991
**  Tehty:            20.12.1991
**  Muutettu:         10.12.1995
**  Mit muutettu:    Muutettu C++ luokaksi
**  Muutettu:         28.12.1995/vl
**  Mit muutettu:    Laitettu uuden jsenen lukeminen toimivaksi
**  Muutettu:         20.01.1997/vl
**  Mit muutettu:    Jsenet erilleen kerhosta
**
**
** Tietorakenne on seuraava:
**
**
**   cNaytto    cKerho
**   ------
**   | o-----> |---|
**   |    |    |   |
**   |    |    |   |
**   |    |    |   |
**   |    |    |---|
**   ------
**
**  Kytss voi olla monta nytt, joista jokainen voi osoittaa samaan
**  tai eri kerhoon.  Nytt huolehtii kaikesta kyttliittymn
**  liittyvst.  Nytt ei tied kerhon rakenteesta muuta kuin, ett
**  kerhon voi lukea levylt ja kirjoittaa levylle sek siihen voi
**  list ja etsi jseni.  Nytt tiet ett on olemassa jsen,
**  muttei se tied jsenen yksityiskohtia.
**
**  Nyttluokka voidaan kirjoittaa uusiksi esimerkiksi Windows-
**  toteuteusta varten.  Tm nyttluokka on ANSI C++:n mukainen,
**  standardi-ptett kyttv versio.
**
** Public-metodeja:
** ================
**  cNaytto(cKerho *p)
**   - alustetaan naytto kayttaen naytossa kerhoa johon p osoittaa
**
**  ~cNaytto()
**   - hvitetn nytt.  Talletetaan jollei ole viel tehty
**
**  void ohjeet(void);
**   - lyhyt tiedotus ohjelmasta
**
**  void kiitokset(void);
**   - poistumiskiitokset
**
**  int  lue_tiedostosta();
**   - luetaan kerhon nimi ja sitten luetaan kerho tiedostosta
**     Luodaan uusi, jollei kerhoa lydy levylt.
**
**  int  talleta();
**   - talletaan kerho levylle (jos siin on muutoksia).
**     vanha kerho j .bak -nimelle
**
**  int  paavalinta();
**   - tulostetaan pmenu ja odotetaan valintaa sek suoritetaan
**     valittu toimenpide.  Toistetaan silmukassa kunnes valitaan
**     poistuminen.
**     Toistaiseksi vasta lisminen toimii Aku Ankalla ja
**     tulostus alkeellisesti nyttn
**
**
*/

#include <iostream>
#include <string>
using namespace std;
#include "naytto.h"
#include "ioali.h"
#include "mjonotpp.h"
#include "dosout.h"

//============================================================================
// Vakiota
//============================================================================
static const char *POIS      = "q";

static const int   OLET_ALKU = 19;
static const int   VAKANEN   = 33;

static const char *TARKENNIN = ".dat";
static const char *VANHATARK = ".bak";

//============================================================================
// Nytn yleinen ksittely:
//============================================================================


void cNaytto::logo(void)
{
  cout << endl;
  cout << "      ###################################" << endl;
  cout << "      #   J  S E N R E K I S T E R I   #" << endl;
  cout << "      #     versio 9.95                 #" << endl;
  cout << "      #     Hannu Hanhi                 #" << endl;
  cout << "      ###################################" << endl;
}


void cNaytto::ohjeet(void)
{
  logo();
  cout << endl;
  cout << "Tll ohjelmalla yllpidetn kerhon jsenrekisteri." << endl;
}


void cNaytto::kiitokset(void)
{
  cout << endl;
  cout << "KIITOS KYTST JA TERVETULOA UUDELLEEN" << endl;
  cout << endl << endl << endl;
}


//============================================================================
// Nytn kerhon tunteva osuus:
//============================================================================

//----------------------------------------------------------------------------
int cNaytto::lue_tiedosto()
/*
** Luetaan kerho levylt.
** Ensin kysytn kerhon nimi.  Jos kerhoa ei ole, utellaan
** listietoja ja luodaan se.
----------------------------------------------------------------------------*/
{
  string tied,nimi; int maksimi;

  do { // Kyselln kunnes tiedosto aukeaa tai luodaan uusi
    cout << endl;
    cout << "Anna kerhon nimi>";  lue_rivi(cin,tied);
    if ( tied == "" ) return ilmoitus("Tiedoston nime ei annettu");

    laita_tarkennin(tied,TARKENNIN);
    if ( onko_tiedostoa(tied) ) return ilmoitus(kerho->lue_tiedostosta(tied));

    cout << "Tiedostoa " << tied << " ei ole!" << endl;
  } while ( kylla_kysymys("Luodaanko uusi tiedosto?") == 0 );

  cout << endl;
  cout << "Anna kerhon koko nimi    >";  lue_rivi(cin,nimi);
  cout << "Anna kerhon maksimi koko >";  lue_rivi(cin,maksimi);

  return ilmoitus(kerho->luo(tied,nimi,maksimi));

}


//----------------------------------------------------------------------------
int cNaytto::talleta()
/*
** Jos kerhossa muutoksia, niin
** Nimetn vanha tiedosto .bak -nimelle ja talletetaan
** nykyinen tiedosto vanhan tilalle.
----------------------------------------------------------------------------*/
{
  logo();
  if ( !kerho->Muutettu() ) return 0;
  int vanhat_pilalla = kerho->TeeBak(VANHATARK);

  if ( ilmoitus(kerho->talleta()) ) return 1;

  cout << endl;
  cout << "Tiedot talletettu tiedostoon "
       << kerho->Tiedoston_nimi() << endl;
  if ( !vanhat_pilalla )
    cout << "Vanhat tiedot tiedostossa    "
         << kerho->Bak_nimi() << endl;

  return 0;
}


//----------------------------------------------------------------------------
int cNaytto::kysy_talleta()
{
  if ( !kerho->Muutettu() ) return 0;
  if ( kylla_kysymys("Talletetaanko tiedot") ) return talleta();
  return 0;
}


//----------------------------------------------------------------------------
void cNaytto::tulosta(ostream &os,const cJasen &jasen)
{
  jasen.tulosta(os);
}


/****************************************************************************/
static int                /*                                                */
kysy_kentta(              /*                                                */
  const string &viesti   ,/* s   Viesti joka tulee nytlle                 */
  string &jono            /* t   Jono johon kentn vastaus luetaan.         */
)
/*
** Funktiolla luetaan vastaus kenttn.
**
** Globaalit: POIS (jono, jolla sytt katkaistaan)
** Sytt:    ptteelt
** Tulostus:  nyttn
** Kutsuu:    lue_jono_oletus
----------------------------------------------------------------------------*/
{
  string apu(jono); int paluu;

  paluu = lue_jono_oletus(viesti,OLET_ALKU,VAKANEN,apu);

  if ( paluu < OLETUS ) return 1;
  if ( apu == POIS ) return 1;

  poista_tyhjat(apu);
  jono = apu;

  return 0;
}


//----------------------------------------------------------------------------
int cNaytto::kysy_tiedot(cJasen &jasen)
{
  cJasen apujasen(jasen);
  string jono;
  int k,kenttia = apujasen.kenttia(), eka = apujasen.eka_kysymys();

  for (k=eka; k<kenttia; k++) {
    jono = apujasen.kentta_jonoksi(k);
    if ( kysy_kentta(apujasen.kysymys(k),jono) ) return 1;
    if ( k == eka && jono == "" ) return 1; /* 1. kys psee pois pelk. ret */
    apujasen.sijoita(k,jono);
  }
  jasen = apujasen;
  return 0;
}


//----------------------------------------------------------------------------
void cNaytto::lisaa_uusi_jasen(char valinta)
/*
** Kysylln uusien jsenien tietoja ja listn jsenet.
----------------------------------------------------------------------------*/
{
  cJasen jasen;

  otsikko(valinta,"Uuden jsenen lisys");

  while ( 1 ) {
    jasen.tyhjenna_omat();
    jasen.rekisteroi();
    cout << endl;
    cout << "Jseni on nyt " << kerho->Jasenia() << "." << endl;
    cout << "Anna uusi nimi muodossa sukunimi etunimi"; // ei endl

    int ei_lisata = 1;
    do {
      cout << endl;                        // Jotta uudella kier. rivi vaiht
      if ( kysy_tiedot(jasen) ) return;

      cout << "Listnk" << endl;
      tulosta(cout,jasen);
      cout << ":";
      if ( kylla_vastaus() )
        ei_lisata = ilmoitus(kerho->lisaa(jasen));
      if ( ei_lisata ) kerho->poista(jasen.Tunnus_nro());
    } while ( ei_lisata );
  }
}


//----------------------------------------------------------------------------
void cNaytto::etsi_jasenen_tiedot(char valinta)
{
  otsikko(valinta,"Etsi jsenen tiedot");
  ei_toimi();
}


//----------------------------------------------------------------------------
void cNaytto::tulosteet(char valinta)
/*
** Tulostetaan kaikki jsenet nytlle.
----------------------------------------------------------------------------*/
{
  otsikko(valinta,"Tulosteet");
  cout << "Toimii toistaiseksi vain nytlle" << endl;
  viivarivi(78,'-');

  for (int i=0; i<kerho->Jasenia(); i++) {
    const cJasen &jasen = kerho->anna_jasen(i);
    tulosta(cout,jasen);
    cout << endl;
  }

  viivarivi(78,'-');
  odota_jotakin();
}


//----------------------------------------------------------------------------
void cNaytto::tietojen_korjailu(char valinta)
{
  otsikko(valinta,"Tietojen korjailu");
  ei_toimi();
}


//----------------------------------------------------------------------------
void cNaytto::paivita_jasenmaksuja(char valinta)
{
  otsikko(valinta,"Pivit jsenmaksuja");
  ei_toimi();
}


//----------------------------------------------------------------------------
void cNaytto::avustus(char valinta)
{
  otsikko(valinta,"Avustus");
  ei_toimi();
}


//----------------------------------------------------------------------------
void cNaytto::paamenu()
{
  cout << endl << endl << endl << endl << endl << endl;
  cout << "Jsenrekisteri"                            << endl;
  cout << "=============="                            << endl;
  cout << ""                                          << endl;
  cout << "Kerhossa " << kerho->Kerhon_nimi() << " on " <<
          kerho->Jasenia() << " jsent."             << endl;
  cout << ""                                          << endl;
  cout << "Valitse:"                                  << endl;
  cout << "   ?  = avustus"                           << endl;
  cout << "   0  = lopetus"                           << endl;
  cout << "   1  = lis uusi jsen"                  << endl;
  cout << "   2  = etsi jsenen tiedot"               << endl;
  cout << "   3  = tulosteet"                         << endl;
  cout << "   4  = tietojen korjailu"                 << endl;
  cout << "   5  = pivit jsenmaksuja"              << endl;
  cout << "   :";
}


//----------------------------------------------------------------------------
int cNaytto::paavalinta()
{
  char nappain;

  while (1) {

    paamenu();

    nappain = odota_nappain("?012345",EI_OLETUSTA,MERKKI_ISOKSI);

    switch (nappain) {
      case '?': avustus(nappain);                     break;
      case '0': return 0;
      case '1': lisaa_uusi_jasen(nappain);            break;
      case '2': etsi_jasenen_tiedot(nappain);         break;
      case '3': tulosteet(nappain);                   break;
      case '4': tietojen_korjailu(nappain);           break;
      case '5': paivita_jasenmaksuja(nappain);        break;
      default : cout << "Nin ei voi kyd!" << endl; return 1;
    }

  }
}


